var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import Crypto from "crypto";
import { EVMAccountAddress, Signature, AESEncryptedString, AESKey, Argon2Hash, EncryptedString, EVMPrivateKey, InitializationVector, SHA256Hash, TokenId, Base64String, InvalidParametersError, } from "@snickerdoodlelabs/objects";
import argon2 from "argon2";
import { ethers } from "ethers";
import { base58 } from "ethers/lib/utils.js";
import { injectable } from "inversify";
import { errAsync, okAsync, ResultAsync } from "neverthrow";
import { ResultUtils } from "neverthrow-result-utils";
import nacl from "tweetnacl";
let CryptoUtils = class CryptoUtils {
    constructor() {
        this.cipherAlgorithm = "aes-256-cbc";
    }
    getNonce(nonceSize = 64) {
        const baseString = Base64String(Crypto.randomBytes(nonceSize).toString("base64").slice(0, nonceSize));
        return okAsync(baseString);
    }
    getTokenId() {
        const buf = Crypto.randomBytes(4);
        const hex = buf.toString("hex");
        const bigInt = BigInt(`0x${hex}`);
        return okAsync(TokenId(bigInt));
        // This older implementation I'm leaving as a comment. It's very clean, but does not
        // work in a service worker, because the polyfills available do not support randomInt().
        // Ye be warned.
        //return okAsync(TokenId(BigInt(Crypto.randomInt(281474976710655))));
    }
    getTokenIds(quantity) {
        if (quantity < 1) {
            return okAsync([]);
        }
        const generateUniqeTokens = (uniqueList = []) => {
            return ResultUtils.combine([...Array(quantity - uniqueList.length)].map(() => {
                return this.getTokenId();
            })).andThen((tokenIds) => {
                const uniqueTokenIds = [...new Set([...uniqueList, ...tokenIds])];
                if (uniqueTokenIds.length !== quantity) {
                    return generateUniqeTokens(uniqueTokenIds);
                }
                return okAsync(uniqueTokenIds);
            });
        };
        return generateUniqeTokens();
    }
    deriveAESKeyFromSignature(signature, salt) {
        // A signature is a hex string, with 65 bytes. We should convert it to a buffer
        const sourceEntropy = this.hexStringToBuffer(signature);
        const saltBuffer = this.hexStringToBuffer(salt);
        const keyBuffer = Crypto.pbkdf2Sync(sourceEntropy, saltBuffer, 100000, 32, "sha256");
        return okAsync(AESKey(keyBuffer.toString("base64")));
    }
    deriveEVMPrivateKeyFromSignature(signature, salt) {
        // A signature is a hex string, with 65 bytes. We should convert it to a buffer
        const sourceEntropy = this.hexStringToBuffer(signature);
        const saltBuffer = this.hexStringToBuffer(salt);
        const keyBuffer = Crypto.pbkdf2Sync(sourceEntropy, saltBuffer, 100000, 32, "sha256");
        return okAsync(EVMPrivateKey(keyBuffer.toString("hex")));
    }
    deriveAESKeyFromEVMPrivateKey(evmKey) {
        // We can generate salt by signing a message
        return this.signMessage("PhoebeIsCute", evmKey).map((signature) => {
            // An EVMPrivateKey is a hex string. We should convert it to a buffer
            const sourceEntropy = this.hexStringToBuffer(evmKey);
            const saltBuffer = this.hexStringToBuffer(signature);
            const keyBuffer = Crypto.pbkdf2Sync(sourceEntropy, saltBuffer, 100000, 32, "sha256");
            return AESKey(keyBuffer.toString("base64"));
        });
    }
    deriveCeramicSeedFromEVMPrivateKey(evmKey) {
        return this.signMessage("VarunWasHere", evmKey).map((signature) => {
            // An EVMPrivateKey is a hex string. We should convert it to a buffer
            const sourceEntropy = this.hexStringToBuffer(evmKey);
            const saltBuffer = this.hexStringToBuffer(signature);
            const keyBuffer = Crypto.pbkdf2Sync(sourceEntropy, saltBuffer, 100000, 32, "sha256");
            return new Uint8Array(keyBuffer.buffer);
        });
    }
    createAESKey() {
        return okAsync(AESKey(Crypto.randomBytes(32).toString("base64")));
    }
    createEthereumPrivateKey() {
        return okAsync(EVMPrivateKey(Crypto.randomBytes(32).toString("hex")));
    }
    getEthereumAccountAddressFromPrivateKey(privateKey) {
        const wallet = new ethers.Wallet(privateKey);
        return EVMAccountAddress(wallet.address);
    }
    verifyEVMSignature(message, signature) {
        const address = EVMAccountAddress(ethers.utils.verifyMessage(message, signature));
        return okAsync(address);
    }
    verifySolanaSignature(message, signature, accountAddress) {
        return okAsync(nacl.sign.detached.verify(Buffer.from(message, "utf-8"), Buffer.from(signature, "hex"), base58.decode(accountAddress)));
    }
    verifyTypedData(domain, types, value, signature) {
        return okAsync(EVMAccountAddress(ethers.utils.verifyTypedData(domain, types, value, signature)));
    }
    encryptString(secret, encryptionKey) {
        return this.getNonce(16).map((nonce) => {
            const iv = InitializationVector(nonce);
            try {
                const cipher = Crypto.createCipheriv(this.cipherAlgorithm, Buffer.from(encryptionKey, "base64"), iv);
                // Encrypt the message
                let encryptedData = cipher.update(secret, "utf8", "base64");
                encryptedData += cipher.final("base64");
                return new AESEncryptedString(EncryptedString(encryptedData), iv);
            }
            catch (e) {
                // This is not ideal error handling, but is better than nothing. At least
                // we will get some logs
                console.error(`Error while encrypting string!`);
                console.error(e);
                return new AESEncryptedString(EncryptedString("THIS IS AN ERROR"), iv);
            }
        });
    }
    decryptAESEncryptedString(encrypted, encryptionKey) {
        try {
            // The decipher function
            const decipher = Crypto.createDecipheriv(this.cipherAlgorithm, Buffer.from(encryptionKey, "base64"), encrypted.initializationVector);
            // decrypt the message
            let decryptedData = decipher.update(encrypted.data, "base64", "utf8");
            decryptedData += decipher.final("utf8");
            return okAsync(decryptedData);
        }
        catch (e) {
            // This is not ideal error handling, but is better than nothing. At least
            // we will get some logs
            console.error(`Error while deciphering encrypted string!`);
            console.error(e);
            return okAsync("THIS IS AN ERROR");
        }
    }
    // public generateKeyPair(): ResultAsync<void, never> {
    // 	const { publicKey, privateKey } = Crypto.generateKeyPairSync("rsa", {
    // 		// The standard secure default length for RSA keys is 2048 bits
    // 		modulusLength: 2048,
    // 	});
    // 	console.log(publicKey);
    // 	return okAsync(undefined);
    // }
    getSignature(owner, types, values) {
        if (types.length !== values.length) {
            return errAsync(new InvalidParametersError("Types and values should have same number of members."));
        }
        const msgHash = ethers.utils.solidityKeccak256([...types], [...values]);
        return ResultAsync.fromSafePromise(owner.signMessage(ethers.utils.arrayify(msgHash))).map((signature) => {
            return Signature(signature);
        });
    }
    signMessage(message, privateKey) {
        const wallet = new ethers.Wallet(privateKey);
        return ResultAsync.fromSafePromise(wallet.signMessage(message)).map((signature) => {
            return Signature(signature);
        });
    }
    signMessageSolana(message, privateKey) {
        return okAsync(Signature(Buffer.from(nacl.sign.detached(Buffer.from(message, "utf8"), base58.decode(privateKey))).toString("hex")));
    }
    signTypedData(domain, types, value, privateKey) {
        const wallet = new ethers.Wallet(privateKey); // TODO, need to specify default provider (https://github.com/ethers-io/ethers.js/issues/2258)
        return ResultAsync.fromSafePromise(wallet._signTypedData(domain, types, value)).map((signature) => {
            return Signature(signature);
        });
    }
    hashStringSHA256(message) {
        const hash = Crypto.createHash("sha256").update(message).digest("base64");
        return okAsync(SHA256Hash(hash));
    }
    hashStringArgon2(message) {
        return ResultAsync.fromSafePromise(argon2.hash(message)).map((hash) => {
            return Argon2Hash(hash);
        });
    }
    verifyHashArgon2(hash, message) {
        return ResultAsync.fromSafePromise(argon2.verify(hash, message));
    }
    xmur3(str) {
        let h = 1779033703 ^ str.length;
        for (let i = 0; i < str.length; i++) {
            (h = Math.imul(h ^ str.charCodeAt(i), 3432918353)),
                (h = (h << 13) | (h >>> 19));
        }
        return function () {
            h = Math.imul(h ^ (h >>> 16), 2246822507);
            h = Math.imul(h ^ (h >>> 13), 3266489909);
            return (h ^= h >>> 16) >>> 0;
        };
    }
    sfc32(a, b, c, d) {
        return function () {
            a >>>= 0;
            b >>>= 0;
            c >>>= 0;
            d >>>= 0;
            let t = (a + b) | 0;
            a = b ^ (b >>> 9);
            b = (c + (c << 3)) | 0;
            c = (c << 21) | (c >>> 11);
            d = (d + 1) | 0;
            t = (t + d) | 0;
            c = (c + t) | 0;
            return (t >>> 0) / 4294967296;
        };
    }
    randomInt(randomFunc, low, high) {
        return Math.floor(randomFunc() * (high - low) + low);
    }
    randomBytes(length, seed) {
        // The seed is any string, we can use that to seed the hash method.
        const hash = this.xmur3(seed);
        // Output four 32-bit hashes to provide the seed for sfc32
        const randFunc = this.sfc32(hash(), hash(), hash(), hash());
        const out = new Uint8Array(length);
        for (let i = 0; i < out.length; i++) {
            out[i] = this.randomInt(randFunc, 0, 256);
        }
        return out;
    }
    hexStringToBuffer(hex) {
        // HexStrings have a nasty habit of SOMETIMES having a 0x prefix but not always.
        // We will correct that
        if (!ethers.utils.isHexString(hex)) {
            const prefixedHex = `0x${hex}`;
            // If it's still not a valid hex string, then it's exception time.
            if (!ethers.utils.isHexString(prefixedHex)) {
                throw new Error(`Invalid hex string ${hex}`);
            }
            return Buffer.from(ethers.utils.arrayify(prefixedHex));
        }
        return Buffer.from(ethers.utils.arrayify(hex));
    }
};
CryptoUtils = __decorate([
    injectable()
], CryptoUtils);
export { CryptoUtils };
//# sourceMappingURL=CryptoUtils.js.map